// Copyright 2023 Azul Systems, Inc.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

package jdk.internal.crac;

import jdk.internal.crac.mirror.impl.CheckpointOpenSocketException;

import java.io.IOException;
import java.net.*;

public abstract class JDKSocketResource extends JDKSocketResourceBase {

    private SocketAddress local;
    private SocketAddress remote;

    public JDKSocketResource(Object owner) {
        super(owner);
    }

    protected abstract SocketAddress localAddress() throws IOException;
    protected abstract SocketAddress remoteAddress() throws IOException;

    @Override
    protected OpenResourcePolicies.Policy findPolicy(boolean isRestore) throws CheckpointOpenSocketException {
        if (!isRestore) {
            try {
                local = localAddress();
            } catch (IOException e) {
                throw new CheckpointOpenSocketException("Cannot find local address for " + owner, e);
            }
            try {
                remote = remoteAddress();
            } catch (IOException e) {
                throw new CheckpointOpenSocketException("Cannot find remote address for " + owner, e);
            }
        }
        var localMatcher = getMatcher(local, "localAddress", "localPort", "localPath");
        var remoteMatcher = getMatcher(remote, "remoteAddress", "remotePort", "remotePath");
        return OpenResourcePolicies.find(isRestore, OpenResourcePolicies.SOCKET,
                params -> localMatcher.test(params) && remoteMatcher.test(params));
    }

    @Override
    protected void reset() {
        // Allow garbage collection
        local = null;
        remote = null;
    }
}
